/* global IntelliPoint:true, _spPageContextInfo, SPClientTemplates, RegisterModuleInit, jQuery */   
(function ($) {
    'use strict';

    // anything that needs to be globally accessible will be attached to this 'namespace'
    window.IntelliPoint = window.IntelliPoint || {};
    IntelliPoint.clientTemplates = IntelliPoint.clientTemplates || {};
    
    // helper because IE doesn't have Array.find
    function arrayFind(array, value, equals) {
        for (var i = 0; i < array.length; i++) {
            if (equals(value, array[i])) return array[i];
        }
    }
    
    /**
     * Instance encapsulating the cascading lookups CSR implementation.
     */
    var cd = {
        //////////////////////////////////////////////////////////////////////
        // BEGIN INTERFACE FOR SetCSRConfig.aspx
        //////////////////////////////////////////////////////////////////////
        name: "cascadingDropdown",
        displayName: "Cascading Lookup",
        canEdit: true,
        types: [ "Lookup" ],
        jsLink: [
            "~sitecollection/Style Library/jquery.js",
            "~sitecollection/Style Library/jquery.spservices.js",
            "~sitecollection/Style Library/intellipoint_clienttemplatesconfig.js",      
            "~sitecollection/Style Library/intellipoint_clienttemplates.js",      
        ],
        
        /**
         * Lauch a dialog to configure a field and call the callback with the
         * configuration node.
         * @param config - the currently saved configuraiton for the field, which
         * is null if the field is new.
         * @param callback - the callback function to which to pass the configuration
         * node when complete.
         * The configuration node (in and out) is just one object from our fields
         * array, so it should look like:
         *     { 
         *         internalName: <fieldInternalName>,
         *         parentColumnInternal: <Internal Name of Parent in this List>,
         *         childColumnInternal: <Internal Name of Child in this List>,
         *         relationshipListParentColumn: <Internal Name of Parent in RelationShip List>,
         *         relationshipListChildColumn: <Internal Name of Child in RelationShip List>,
         *         relationshipList: <Title of the Relationship List>,
         *         relationshipWebURL: <URL of the Web in which the Relationship List Resides>
         *     }
         */
        configure: function(internalName, config, callback) {
            // create the dialog ui
            if($("#cascadingDropdownDialog").length === 0) {
                var modal = $("<div/>").attr("id", "cascadingDropdownDialog");
                
                modal.append($("<span/>").addClass("label").text("Parent Column Internal Name:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "parentColumnInternal")));
                modal.append("<br/>");
                    
                modal.append($("<span/>").addClass("label").text("Child Column Internal Name:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "childColumnInternal")));
                modal.append("<br/>");
                
                modal.append($("<span/>").addClass("label").text("Relationship List Title:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "relationshipList")));
                modal.append("<br/>");
                                    
                modal.append($("<span/>").addClass("label").text("Relationship Parent Column Internal Name:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "relationshipListParentColumn")));
                modal.append("<br/>");
                                    
                modal.append($("<span/>").addClass("label").text("Relationship Child Column Internal Name:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "relationshipListChildColumn")));
                modal.append("<br/>");
                                    
                modal.append($("<span/>").addClass("label").text("Relationship Web URL:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "relationshipWebURL")));
                modal.append("<br/>");
                                    
                $("#dialogs").append(modal);
            }
            
            // initialize the dialog inputs
            if(config) {
                if(config.webURL) {
                	$("#relationshipWebURL").val(config.relationshipWebURL);
                }
                else {
                    $("#relationshipWebURL").val(_spPageContextInfo.siteServerRelativeUrl);
                }
                $("#parentColumnInternal").val(config.parentColumnInternal);
                $("#childColumnInternal").val(config.childColumnInternal);
                $("#relationshipList").val(config.relationshipList);
                $("#relationshipListParentColumn").val(config.relationshipListParentColumn);
                $("#relationshipListChildColumn").val(config.relationshipListChildColumn);
            }
            else {
                $("#relationshipWebURL").val(_spPageContextInfo.siteServerRelativeUrl);
                $("#parentColumnInternal").val("");
                $("#childColumnInternal").val("");
                $("#relationshipList").val("");
                $("#relationshipListParentColumn").val("");
                $("#lookupListTitle").val("");
            }
            
            // launch the dialog
            $("#cascadingDropdownDialog").dialog({
            	title: "Add/Edit Autocomplete Field",
                modal: true,
                width: 700,
                buttons: [
                	{
                        text: "Ok",
                        click: function() {
                            callback({
                                internalName: internalName,
                                parentColumnInternal: $("#parentColumnInternal").val(),
                                childColumnInternal: $("#childColumnInternal").val(),
                                relationshipList: $("#relationshipList").val(),
                                relationshipListParentColumn: $("#relationshipListParentColumn").val(),
                                relationshipListChildColumn: $("#relationshipListChildColumn").val(),
                                relationshipWebURL: $("#relationshipWebURL").val(),
                            });
                            $(this).dialog("close");
                        }
                    },
                    {
                    	text: "Cancel",
                        click: function() {
                            $(this).dialog("close");
                        }
                    }
                ],
            });
        },
                
        /**
         * Get an array of objects describing the callbacks for a field.
         * @fields - the field configuration for this CSR implementation.
         * @return [object] where the objects look like:
         * {
         *     internalName: <name of field>,
         *     OnPreRender: <callback>,
         *     OnPostRender: <callback>,
         *     NewForm: <callback>,
         *     EditForm: <callback>,
         *     DisplayForm: <callback>,
         *     View: <callback>
         * }
         * Any defined callbacks will be included in the overrides object passed
         * to RegisterTemplateOverrides. Unlike normal CSR, OnPreRender and OnPostRender
         * will only be called for the field they're configured on, not all fields
         * on the form (this is usually what I want, and it normalizes things for
         * undocumented changes in the way these callbacks are handled on office 365).
         */
        getClientTemplates: function (fields) {
        	var result = [];
            $.each(fields, function(i,v) {
                result.push({
                    internalName: v.internalName,
                    NewForm: cd.render,
                    EditForm: cd.render,
                });
            });
            return result;
        },

        //////////////////////////////////////////////////////////////////////
        // END INTERFACE FOR SetCSRConfig.aspx
        //////////////////////////////////////////////////////////////////////

        // this will be the render method we pass in overrides
        render: function(ctx) {
            // get the form context so we can register some callbacks
            var formCtx = SPClientTemplates.Utility.GetFormContextForCurrentField(ctx);

            // get the OOB default rendering
            var html = cd.getDefaultRendering(ctx);

            // register an init callback for our field
            formCtx.registerInitCallback(formCtx.fieldName, function() {
                // find the current config element
                var config = arrayFind(
                    IntelliPoint.clientTemplatesConfig.cascadingDropdown,
                    formCtx.fieldName,
                    function(v, o) {
                        return v === o.internalName;
                    });

                // get the list schema for all fields
                var schema = window[ctx.FormUniqueId + "FormCtx"].ListSchema;

                // get the display names SPServices wants from the internal names
                // in our configuration
                if (config.parentColumnInternal) {
                    config.parentColumn = schema[config.parentColumnInternal].Title;
                }
                if (config.childColumnInternal) {
                    config.childColumn = schema[config.childColumnInternal].Title;
                }

                // if relationshipWebURL is not specified, assume the root web
                if (!config.relationshipWebURL) {
                    config.relationshipWebURL = _spPageContextInfo.siteServerRelativeUrl;
                }

                // apply SPCascadeDropdowns to our configuration
                $().SPServices.SPCascadeDropdowns(config);
            });

            // need to do get value callback because lookup updates it's value in an onchange
            // handler declared in init callback, which we've overridden
            formCtx.registerGetValueCallback(formCtx.fieldName, function() {
                var val = $("select[id^='" + formCtx.fieldName + "_']").val();
                var text = $("select[id^='" + formCtx.fieldName + "_'] option:selected").text();
                return (val === '0' || val === '' ? '' : val + ";#" + text);
            });

            // return the OOB default rendering
            return html;
        },

        getDefaultRendering: function(ctx) {
            var templatesByType = SPClientTemplates._defaultTemplates.Fields.default.all.all;
            var currentTemplates = templatesByType[ctx.CurrentFieldSchema.Type];
            var currentRenderFunc = currentTemplates[ctx.BaseViewID];
            return currentRenderFunc(ctx);
        }
    };
    // attaching to client templates exposes this implementation to the utility page and
    // the overrides construction
    IntelliPoint.clientTemplates.cascadingDropdown = cd;
    
    // this structure is the only thing that needs to be modified to override more or different fields
    var autoCompleteFields = IntelliPoint.clientTemplatesConfig.autocomplete.map(function(f) {
        return f.internalName;
    });

    /**
     * Instance encapsulating the autocomplete CSR implementation.
     */
    var ac = {
        //////////////////////////////////////////////////////////////////////
        // BEGIN INTERFACE FOR SetCSRConfig.aspx
        //////////////////////////////////////////////////////////////////////
        name: "autocomplete",
        displayName: "Autocomplete",
        canEdit: true,
        types: [ "Text" ],
        jsLink: [
            "~sitecollection/Style Library/jquery.js",
            "~sitecollection/Style Library/jquery-ui.js",
            "~sitecollection/Style Library/jquery.spservices.js",
            "~sitecollection/Style Library/intellipoint_clienttemplatesconfig.js",      
            "~sitecollection/Style Library/intellipoint_clienttemplates.js",      
        ],
        
        /**
         * Lauch a dialog to configure a field and call the callback with the
         * configuration node.
         * @param config - the currently saved configuraiton for the field, which
         * is null if the field is new.
         * @param callback - the callback function to which to pass the configuration
         * node when complete.
         * The configuration node (in and out) is just one object from our fields
         * array, so it should look like:
         *     { 
         *         internalName: <fieldInternalName>,
         *         webURL: <URL of the web where the lookup list lives>,
         *         lookupList: <title of the lookup list>,
         *         lookupField: <internal name of the field>,
         *     }
         */
        configure: function(internalName, config, callback) {
            // create the dialog ui
            if($("#autocompleteDialog").length === 0) {
                var modal = $("<div/>").attr("id", "autocompleteDialog");
                
                modal.append($("<span/>").addClass("label").text("Lookup List Title:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "lookupListTitle")));
                modal.append("<br/>");
                    
                modal.append($("<span/>").addClass("label").text("Lookup Field Internal Name:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "lookupFieldName")));
                modal.append("<br/>");

                modal.append($("<span/>").addClass("label").text("Lookup Web URL:"));
                modal.append($("<span/>").addClass("input").append(
                    $("<input/>").attr("id", "lookupWebUrlInput")));
                modal.append("<br/>");
                                    
                $("#dialogs").append(modal);
            }
            
            // initialize the dialog inputs
            if(config) {
                if(config.webURL) {
                	$("#lookupWebUrlInput").val(config.webURL);
                }
                else {
                    $("#lookupWebUrlInput").val(_spPageContextInfo.siteServerRelativeUrl);
                }
                $("#lookupListTitle").val(config.lookupList);
                $("#lookupFieldName").val(config.lookupField);
            }
            else {
                $("#lookupWebUrlInput").val(_spPageContextInfo.siteServerRelativeUrl);
                $("#lookupListTitle").val("");
                $("#lookupFieldName").val("");
            }
            
            // launch the dialog
            $("#autocompleteDialog").dialog({
            	title: "Add/Edit Autocomplete Field",
                modal: true,
                width: 600,
                buttons: [
                	{
                        text: "Ok",
                        click: function() {
                            callback({
                                internalName: internalName,
                                webURL: $("#lookupWebUrlInput").val(),
                                lookupList: $("#lookupListTitle").val(),
                                lookupField: $("#lookupFieldName").val(),
                            });
                            $(this).dialog("close");
                        }
                    },
                    {
                    	text: "Cancel",
                        click: function() {
                            $(this).dialog("close");
                        }
                    }
                ],
            });
        },
        
        /**
         * Get an array of objects describing the callbacks for a field.
         * @fields - the field configuration for this CSR implementation.
         * @return [object] where the objects look like:
         * {
         *     internalName: <name of field>,
         *     OnPreRender: <callback>,
         *     OnPostRender: <callback>,
         *     NewForm: <callback>,
         *     EditForm: <callback>,
         *     DisplayForm: <callback>,
         *     View: <callback>
         * }
         * Any defined callbacks will be included in the overrides object passed
         * to RegisterTemplateOverrides. Unlike normal CSR, OnPreRender and OnPostRender
         * will only be called for the field they're configured on, not all fields
         * on the form (this is usually what I want, and it normalizes things for
         * undocumented changes in the way these callbacks are handled on office 365).
         */
        getClientTemplates: function (fields) {
        	var result = [];
            $.each(fields, function(i,v) {
                result.push({
                    internalName: v.internalName,
                    OnPreRender: ac.preRender,
                    OnPostRender: ac.postRender,
                });
            });
            return result;
        },
        
        //////////////////////////////////////////////////////////////////////
        // END INTERFACE FOR SetCSRConfig.aspx
        //////////////////////////////////////////////////////////////////////
        
        addedCss: false,

        /**
         * The callback for pre render. 
         * This method just imports the jquery ui css into the page.
         */
        preRender: function(ctx) {
            if (!ac.addedCss) {
                $("head").append(
                    "<link rel='stylesheet' type='text/css' href='" +
                    _spPageContextInfo.siteAbsoluteUrl +
                    "/Style Library/jquery-ui.css'>");
                ac.addedCss = true;
            }
        },

        /**
         * The callback for post render. 
         * This method calls the Lists web service to get the autocomplete data, and then
         * sets up jquery ui autocomplete on the input.
         */
        postRender: function(ctx) {
            var schema = ctx.ListSchema.Field[0];
            if ($.inArray(schema.Name, autoCompleteFields) > -1) {
                // get the field configuration by internal name
                var config = arrayFind(
                    IntelliPoint.clientTemplatesConfig.autocomplete,
                    schema.Name, 
                    function(v, o) {
                        return v === o.internalName;
                    });

                // find the HTML input by naming convention (id starts with <InternalName>_<ID>)
                var $input = $("[id^='" + schema.Name + "_" + schema.Id + "']");

                // setup view fields and caml for the query of the lookup list
                var viewFields = "<ViewFields><FieldRef Name='" + 
                    config.lookupField + "' /></ViewFields>";
                var query =
                    "<Query><OrderBy>" +
                    "<FieldRef Name='" + config.lookupField + "' Ascending='True' />" +
                    "</OrderBy></Query>";

                // if the web url isn't in the config, assume the root web
                if (!config.webURL) {
                    config.webURL = _spPageContextInfo.siteServerRelativeUrl;
                }

                // load the autocomplete data from the lookup list and initialize autocomplete
                $().SPServices({
                    webURL: config.webURL,
                    operation: "GetListItems",
                    async: true,
                    listName: config.lookupList,
                    CAMLViewFields: viewFields,
                    CAMLQuery: query,
                    completefunc: function(xData) {
                        var autocompleteData = [];
                        $(xData.responseXML).SPFilterNode("z:row").each(function() {
                            // push the results into an array
                            autocompleteData.push($(this).attr("ows_" + config.lookupField));
                        });

                        // initialize jquery ui autocomplete with the data
                        if (autocompleteData.length > 0) {
                            $input.autocomplete({
                                source: autocompleteData,
                                minLength: 2
                            });
                        }
                    }
                });
            }
        }
    };
    // attaching to client templates exposes this implementation to the utility page and
    // the overrides construction
    IntelliPoint.clientTemplates.autocomplete = ac;


    if(typeof(SPClientTemplates) !== "undefined" && 
        typeof(IntelliPoint.clientTemplatesConfig) !== "undefined") {
    
        // maps of field internal name to pre/post render callback respectively
        var preRenderers = {};
        var postRenderers = {};
        
        // create an overrides object
        var overrides = {
            Templates: {
            	Fields: {}
            }
        };
        
        // Loop through each implementation and based on the configuration construct
        // the Template.Fields. Also save off any pre and post render overrides, which
        // will be called back from our centralized pre and post render handlers below.
        var keys = Object.keys(IntelliPoint.clientTemplates);
        $.each(keys, function(i, template) {
            var templateImpl = IntelliPoint.clientTemplates[template];
            if(templateImpl) {
                var templateConfig = IntelliPoint.clientTemplatesConfig[template];
                if(templateConfig) {
                    var implOverrides = templateImpl.getClientTemplates(templateConfig);
                    $.each(implOverrides, function(j, o) {
                        if(o.View || o.DisplayForm || o.NewForm || o.EditForm) {
                            var newOverrides = {};

                            if(o.View) newOverrides.View = o.View;
                            if(o.NewForm) newOverrides.NewForm = o.NewForm;
                            if(o.EditForm) newOverrides.EditForm = o.EditForm;
                            if(o.View) newOverrides.DisplayForm = o.DisplayForm;

                            // collect all field overrides in our overrides instance
                            overrides.Templates.Fields[o.internalName] = newOverrides;
                        }

                        // collect up pre renderers to call back later from our pre render method
                        if(o.OnPreRender) {
                            preRenderers[o.internalName] = o.OnPreRender;
                        }

                        // collect up post renderers to call back later from our post render method
                        if(o.OnPostRender) {
                            postRenderers[o.internalName] = o.OnPostRender;
                        }
                    });
                }
            }
        });

        // if we have pre renderers, define overrides.OnPreRender to call them as appropriate
        if(Object.keys(preRenderers).length > 0) {
            overrides.OnPreRender = function(ctx) {
                var internalName = ctx.ListSchema.Field[0].Name;
                if(preRenderers[internalName] && typeof(preRenderers[internalName]) === "function") {
                    preRenderers[internalName](ctx);
                }
            };
        }
    
        // if we have post renderers, define overrides.OnPostRender to call them as appropriate
        if(Object.keys(postRenderers).length > 0) {
            overrides.OnPostRender = function(ctx) {
                var internalName = ctx.ListSchema.Field[0].Name;
                if(postRenderers[internalName] && typeof(postRenderers[internalName]) === "function") {
                    postRenderers[internalName](ctx);
                }
            };
        }
    
        // register templates now for non-MDS and full page loads
        SPClientTemplates.TemplateManager.RegisterTemplateOverrides(overrides);

        // register template overrides for partial page loads if MDS is enabled
        if (typeof _spPageContextInfo !== 'undefined' && _spPageContextInfo !== null) {
            var base = _spPageContextInfo.siteServerRelativeUrl;
            var url = (base === '/' ? "" : base).toLowerCase();

            RegisterModuleInit(url + '/style%20library/intellipoint_clienttemplates.js', function() {
                SPClientTemplates.TemplateManager.RegisterTemplateOverrides(overrides);
            });
        }
    }
})(jQuery);
